from question import SurveyElement
from utils import node
from errors import PyXFormError


class Section(SurveyElement):
    def validate(self):
        super(Section, self).validate()
        for element in self.children:
            element.validate()
        self._validate_uniqueness_of_element_names()

    # there's a stronger test of this when creating the xpath
    # dictionary for a survey.
    def _validate_uniqueness_of_element_names(self):
        element_slugs = []
        for element in self.children:
            if element.name in element_slugs:
                raise PyXFormError(
                    "There are two survey elements named '%s' in the section named '%s'." % (element.name, self.name)
                    )
            element_slugs.append(element.name)

    def xml_instance(self, **kwargs):
        """
        Creates an xml representation of the section
        """
        result = node(self.name, **kwargs)
        for child in self.children:
            result.appendChild(child.xml_instance())
        return result

    def xml_control(self):
        """
        Ideally, we'll have groups up and rolling soon, but for now
        let's just return a list of controls from all the children of
        this section.
        """
        return [e.xml_control() for e in self.children if e.xml_control() is not None]


class RepeatingSection(Section):
    def xml_control(self):
        """
        <group>
        <label>Fav Color</label>
        <repeat nodeset="fav-color">
          <select1 ref=".">
            <label ref="jr:itext('fav')" />
            <item><label ref="jr:itext('red')" /><value>red</value></item>
            <item><label ref="jr:itext('green')" /><value>green</value></item>
            <item><label ref="jr:itext('yellow')" /><value>yellow</value></item>
          </select1>
        </repeat>
        </group>
        """
        control_dict = self.control.copy()
        jrcount = control_dict.get('jr:count')
        if jrcount:
            survey = self.get_root()
            control_dict['jr:count'] = survey.insert_xpaths(jrcount)
        repeat_node = node(u"repeat", nodeset=self.get_xpath(), **control_dict)

        for n in Section.xml_control(self):
            repeat_node.appendChild(n)

        label = self.xml_label()
        if label:
            return node(
                u"group", self.xml_label(), repeat_node,
                ref=self.get_xpath()
                )
        return node(u"group", repeat_node, ref=self.get_xpath())
    #I'm anal about matching function signatures when overriding a function, but there's no reason for kwargs to be an argument
    def xml_instance(self, **kwargs):
        kwargs = {"jr:template": ""} #It might make more sense to add this as a child on initialization
        return super(RepeatingSection, self).xml_instance(**kwargs)

class GroupedSection(Section):
#    I think this might be a better place for the table-list stuff, however it doesn't allow for as good of validation as putting it in xls2json
#    def __init__(self, **kwargs):
#        control = kwargs.get(u"control")
#        if control:
#            appearance = control.get(u"appearance")
#            if appearance is u"table-list":
#                print "HI"
#                control[u"appearance"] = "field-list"
#                kwargs["children"].insert(0, kwargs["children"][0])
#        super(GroupedSection, self).__init__(kwargs)
        
    def xml_control(self):
        control_dict = self.control
        
        if control_dict.get("bodyless"):
            return None
            
        children = []
        attrs = { 'ref':self.get_xpath() }
        
        if 'label' in self and len(self['label']) > 0:
            children.append(self.xml_label())
        for n in Section.xml_control(self):
            children.append(n)
        
        if u"appearance" in control_dict:
            attrs['appearance'] = control_dict['appearance']
        
        return node(u"group", *children, **attrs)

    def to_json_dict(self):
        # This is quite hacky, might want to think about a smart way
        # to approach this problem.
        result = super(GroupedSection, self).to_json_dict()
        result[u"type"] = u"group"
        return result



"""
odk_validate.py
A python wrapper around ODK Validate
"""
import os
import sys
from subprocess import Popen, PIPE
import threading
import signal

CURRENT_DIRECTORY = os.path.dirname(os.path.realpath(__file__))
ODK_VALIDATE_JAR = os.path.join(CURRENT_DIRECTORY, "ODK_Validate.jar")


#Adapted from:
#http://betabug.ch/blogs/ch-athens/1093
def run_popen_with_timeout(command, timeout):
    """
    Run a sub-program in subprocess.Popen, pass it the input_data,
    kill it if the specified timeout has passed.
    returns a tuple of resultcode, timeout, stdout, stderr
    """
    kill_check = threading.Event()

    def _kill_process_after_a_timeout(pid):
        os.kill(pid, signal.SIGTERM)
        kill_check.set()  # tell the main routine that we had to kill
        # use SIGKILL if hard to kill...
        return
    p = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
    pid = p.pid
    watchdog = threading.Timer(
        timeout, _kill_process_after_a_timeout, args=(pid, ))
    watchdog.start()
    (stdout, stderr) = p.communicate()
    watchdog.cancel()  # if it's still waiting to run
    timeout = kill_check.isSet()
    kill_check.clear()
    return (p.returncode, timeout, stdout, stderr)


def check_xform(path_to_xform):
    """
    Returns an array of warnings if the form is valid.
    Throws an exception if it is not
    """
    #resultcode indicates validity of the form
    #timeout indicates whether validation ran out of time to complete
    #stdout is not used because it has some warnings that always
    #appear and can be ignored.
    #stderr is treated as a warning if the form is valid or an error
    #if it is invalid.
    returncode, timeout, stdout, stderr = run_popen_with_timeout(
        ["java", "-jar", ODK_VALIDATE_JAR, path_to_xform], 100)
    warnings = []

    if timeout:
        return ["XForm took to long to completely validate."]
    else:
        if returncode > 0:  # Error invalid
            raise Exception('ODK Validate Errors:\n' + stderr)
        elif returncode == 0:
            if stderr:
                warnings.append('ODK Validate Warnings:\n' + stderr)
            return warnings
        elif returncode < 0:
            return ["Bad return code from ODK Validate."]

if __name__ == '__main__':
    print __doc__
    check_xform(sys.argv[1])



from xml.dom.minidom import Text, Element, parseString
import re
import codecs
import json
import copy

SEP = "_"

# http://www.w3.org/TR/REC-xml/
TAG_START_CHAR = r"[a-zA-Z:_]"
TAG_CHAR = r"[a-zA-Z:_0-9\-.]"
XFORM_TAG_REGEXP = "%(start)s%(char)s*" % {
    "start": TAG_START_CHAR,
    "char": TAG_CHAR
    }


def is_valid_xml_tag(tag):
    """
    Use a regex to see if there are any invalid characters (i.e. spaces).
    """
    return re.search(r"^" + XFORM_TAG_REGEXP + r"$", tag)

def node(tag, *args, **kwargs):
    """
    tag -- a XML tag
    args -- an array of children to append to the newly created node
            or if a unicode arg is supplied it will be used to make a text node
    returns a xml.dom.minidom.Element
    """
    result = Element(tag)
    unicode_args = [u for u in args if type(u) == unicode]
    assert len(unicode_args) <= 1
    parsedString = False
    #kwargs is an xml attribute dictionary, here we convert it to a xml.dom.minidom.Element
    for k, v in kwargs.iteritems():
        if k == 'toParseString':
            if v == True and len(unicode_args) == 1:
                parsedString = True
                #Add this header string so parseString can be used?
                s = u'<?xml version="1.0" ?><'+tag+'>' + unicode_args[0] + u'</'+tag+'>'
                node = parseString(s.encode("utf-8")).documentElement
                #Move node's children to the result Element discarding node's root
                for child in node.childNodes:
                    result.appendChild(copy.deepcopy(child))
        else:
            result.setAttribute(k, v)

    if len(unicode_args) == 1 and not parsedString:
        text_node = Text()
        text_node.data = unicode_args[0]
        result.appendChild(text_node)
    for n in args:
        if type(n) == int or type(n) == float or type(n) == str:
            text_node = Text()
            text_node.data = unicode(n)
            result.appendChild(text_node)
        elif type(n) is not unicode:
            try:
                result.appendChild(n)
            except:
                raise Exception(type(n), n)
    return result


def get_pyobj_from_json(str_or_path):
    """
    This function takes either a json string or a path to a json file,
    it loads the json into memory and returns the corresponding Python
    object.
    """
    try:
        # see if treating str_or_path as a path works
        fp = codecs.open(str_or_path, mode="r", encoding="utf-8")
        doc = json.load(fp, encoding="utf-8")
    except:
        # if it doesn't work load the text
        doc = json.loads(str_or_path)
    return doc



"""
xls2xform converts properly formatted Excel documents into XForms for
use with ODK Collect.
"""
import os, sys
import xls2json
import builder

if __name__ == '__main__':
    argv = sys.argv
    if len(argv) < 3:
        print __doc__
        print 'Usage:'
        print argv[0] + ' path_to_XLSForm output_path'
    else:
        warnings = []
        json_survey = xls2json.parse_file_to_json(argv[1], warnings=warnings)
        survey = builder.create_survey_element_from_dict(json_survey)
        #Setting validate to false will cause the form not to be processed by ODK Validate.
        #This may be desirable since ODK Validate requires launching a subprocess that runs some java code.
        survey.print_xform_to_file(argv[2], validate=True, warnings=warnings)
        for w in warnings:
            print w
        print 'Conversion complete!'


from xls2json import QuestionTypesReader, print_pyobj_to_json

def generate_new_dict():
    """
    This is just here incase there is ever any need to generate the question type dictionary from all.xls again.
    It shouldn't be called as part of any application.
    """
    path_to_question_types = "/home/nathan/aptana-workspace/pyxform/pyxform/question_types/all.xls"
    json_dict = QuestionTypesReader(path_to_question_types).to_json_dict()
    print_pyobj_to_json(json_dict, 'new_quesiton_type_dict.json')

QUESTION_TYPE_DICT = \
{
    "q picture": {
        "control": {
            "tag": "upload", 
            "mediatype": "image/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "photo": {
        "control": {
            "tag": "upload", 
            "mediatype": "image/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "add date time prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "dateTime"
        }
    }, 
    "add audio prompt": {
        "control": {
            "tag": "upload", 
            "mediatype": "audio/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "q date time": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "dateTime"
        }
    }, 
    "phonenumber": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "phonenumber"
        }
    }, 
    "get start time": {
        "bind": {
            "jr:preload": "timestamp", 
            "type": "dateTime", 
            "jr:preloadParams": "start"
        }
    }, 
    "add select multiple prompt using": {
        "control": {
            "tag": "select"
        }, 
        "bind": {
            "type": "select"
        }
    }, 
    "add note prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "readonly": "true()", 
            "type": "string"
        }
    }, 
    "calculate": {
        "bind": {
            "type": "string"
        }
    }, 
    "acknowledge": {
        "control": {
            "tag": "trigger"
        }, 
        "bind": {
            "type": "string"
        }
    }, 
    "location": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "geopoint"
        }
    }, 
    "text": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "string"
        }
    }, 
    "select all that apply from": {
        "control": {
            "tag": "select"
        }, 
        "bind": {
            "type": "select"
        }
    }, 
    "simserial": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "simserial"
        }
    }, 
    "string": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "string"
        }
    }, 
    "q string": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "string"
        }
    }, 
    "imei": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "deviceid"
        }
    }, 
    "integer": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int"
        }
    }, 
    "datetime": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "dateTime"
        }
    }, 
    "q note": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "readonly": "true()", 
            "type": "string"
        }
    }, 
    "subscriber id": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "subscriberid"
        }
    }, 
    "decimal": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "decimal"
        }
    }, 
    "dateTime": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "dateTime"
        }
    }, 
    "q audio": {
        "control": {
            "tag": "upload", 
            "mediatype": "audio/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "q geopoint": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "geopoint"
        }
    }, 
    "q image": {
        "control": {
            "tag": "upload", 
            "mediatype": "image/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "get today": {
        "bind": {
            "jr:preload": "date", 
            "type": "date", 
            "jr:preloadParams": "today"
        }
    }, 
    "video": {
        "control": {
            "tag": "upload", 
            "mediatype": "video/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "q acknowledge": {
        "control": {
            "tag": "trigger"
        }, 
        "bind": {
            "type": "string"
        }
    }, 
    "add video prompt": {
        "control": {
            "tag": "upload", 
            "mediatype": "video/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "number of days in last month": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int", 
            "constraint": "0 <= . and . <= 31"
        }, 
        "hint": "Enter a number 0-31."
    }, 
    "get sim id": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "simserial"
        }
    }, 
    "q location": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "geopoint"
        }
    }, 
    "select one": {
        "control": {
            "tag": "select1"
        }, 
        "bind": {
            "type": "select1"
        }
    }, 
    "add image prompt": {
        "control": {
            "tag": "upload", 
            "mediatype": "image/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "select all that apply": {
        "control": {
            "tag": "select"
        }, 
        "bind": {
            "type": "select"
        }
    }, 
    "get end time": {
        "bind": {
            "jr:preload": "timestamp", 
            "type": "dateTime", 
            "jr:preloadParams": "end"
        }
    }, 
    "barcode": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "barcode"
        }
    }, 
    "q video": {
        "control": {
            "tag": "upload", 
            "mediatype": "video/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "geopoint": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "geopoint"
        }, 
        "hint": "GPS coordinates can only be collected when outside."
    }, 
    "select multiple from": {
        "control": {
            "tag": "select"
        }, 
        "bind": {
            "type": "select"
        }
    }, 
    "end time": {
        "bind": {
            "jr:preload": "timestamp", 
            "type": "dateTime", 
            "jr:preloadParams": "end"
        }
    }, 
    "device id": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "deviceid"
        }
    },
    "subscriberid": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "subscriberid"
        }
    }, 
    "q barcode": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "barcode"
        }
    }, 
    "q select": {
        "control": {
            "tag": "select"
        }, 
        "bind": {
            "type": "select"
        }
    }, 
    "select one using": {
        "control": {
            "tag": "select1"
        }, 
        "bind": {
            "type": "select1"
        }
    }, 
    "image": {
        "control": {
            "tag": "upload", 
            "mediatype": "image/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "q int": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int"
        }
    }, 
    "add text prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "string"
        }
    }, 
    "add date prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "date"
        }
    }, 
    "q calculate": {
        "bind": {
            "type": "string"
        }
    }, 
    "start": {
        "bind": {
            "jr:preload": "timestamp", 
            "type": "dateTime", 
            "jr:preloadParams": "start"
        }
    }, 
    "trigger": {
        "control": {
            "tag": "trigger"
        }
    }, 
    "add acknowledge prompt": {
        "control": {
            "tag": "trigger"
        }, 
        "bind": {
            "type": "string"
        }
    }, 
    "percentage": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int", 
            "constraint": "0 <= . and . <= 100"
        }
    }, 
    "get phone number": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "phonenumber"
        }
    }, 
    "today": {
        "bind": {
            "jr:preload": "date", 
            "type": "date", 
            "jr:preloadParams": "today"
        }
    }, 
    "gps": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "geopoint"
        }, 
        "hint": "GPS coordinates can only be collected when outside."
    }, 
    "q date": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "date"
        }
    }, 
    "sim id": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "simserial"
        }
    }, 
    "add decimal prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "decimal"
        }
    }, 
    "number of days in last six months": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int", 
            "constraint": "0 <= . and . <= 183"
        }, 
        "hint": "Enter a number 0-183."
    }, 
    "deviceid": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "deviceid"
        }
    }, 
    "int": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int"
        }
    }, 
    "add barcode prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "barcode"
        }
    }, 
    "select multiple using": {
        "control": {
            "tag": "select"
        }, 
        "bind": {
            "type": "select"
        }
    }, 
    "q decimal": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "decimal"
        }
    }, 
    "end": {
        "bind": {
            "jr:preload": "timestamp", 
            "type": "dateTime", 
            "jr:preloadParams": "end"
        }
    }, 
    "add calculate prompt": {
        "bind": {
            "type": "string"
        }
    }, 
    "add dateTime prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "dateTime"
        }
    }, 
    "note": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "readonly": "true()", 
            "type": "string"
        }
    }, 
    "add location prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "geopoint"
        }
    }, 
    "get subscriber id": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "subscriberid"
        }
    }, 
    "phone number": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "string", 
            "constraint": "regex(., '^\\d*$')"
        }, 
        "hint": "Enter numbers only."
    }, 
    "get device id": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "deviceid"
        }
    }, 
    "add integer prompt": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int"
        }
    }, 
    "q dateTime": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "dateTime"
        }
    }, 
    "date": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "date"
        }
    }, 
    "q select1": {
        "control": {
            "tag": "select1"
        }, 
        "bind": {
            "type": "select1"
        }
    }, 
    "start time": {
        "bind": {
            "jr:preload": "timestamp", 
            "type": "dateTime", 
            "jr:preloadParams": "start"
        }
    }, 
    "number of days in last year": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "int", 
            "constraint": "0 <= . and . <= 365"
        }, 
        "hint": "Enter a number 0-365."
    }, 
    "date time": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "dateTime"
        }
    }, 
    "time": {
        "control": {
            "tag": "input"
        }, 
        "bind": {
            "type": "time"
        }
    }, 
    "audio": {
        "control": {
            "tag": "upload", 
            "mediatype": "audio/*"
        }, 
        "bind": {
            "type": "binary"
        }
    }, 
    "add select one prompt using": {
        "control": {
            "tag": "select1"
        }, 
        "bind": {
            "type": "select1"
        }
    },
    "hidden": {
        "bind": {
            "type": "string"
        }
    },
    "uri:subscriberid": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "uri:subscriberid"
        }
    },
    "uri:phonenumber": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "uri:phonenumber"
        }
    },
    "uri:simserial": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "uri:simserial"
        }
    },
    "uri:deviceid": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "uri:deviceid"
        }
    }, 
    "uri:username": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "uri:username"
        }
    }, 
    "uri:email": {
        "bind": {
            "jr:preload": "property", 
            "type": "string", 
            "jr:preloadParams": "uri:email"
        }
    }, 
}

#import os
#class QuestionTypeDictionary(dict):
#    """
#    A dictionary parsed from an xls file that defines question types.
#    """
#    def __init__(self, file_name="base"):
#        # Right now we're using an excel file to describe question
#        # types we will use in creating XForms, we'll switch over to
#        # json soon.
#        self._name = file_name
#        path_to_this_file = os.path.abspath(__file__)
#        path_to_this_dir = os.path.dirname(path_to_this_file)
#        path_to_question_types = os.path.join(
#            path_to_this_dir,
#            "question_types",
#            "%s.xls" % file_name
#            )
#        excel_reader = QuestionTypesReader(path_to_question_types)
#        for k, v in excel_reader.to_json_dict().iteritems():
#            self[k] = v
#
#    def get_definition(self, question_type_str):
#        return self.get(question_type_str, {})
#
#DEFAULT_QUESTION_TYPE_DICTIONARY = QuestionTypeDictionary("all")
